; JOYSTICK READER (REVISION 1)

;  0: Reads Joystick Port 1 and set Register Pair HL

;     to reflect the status of the port.

;     Want to include in Revision 1 reading of Port 2, so the

;     game can read the Reset Button.

;  1: Now can read both ports.  Revision 0 never really existed 

;     as reading both ports was much easier than I thougt.



; REQUIREMENTS:

; None



; CHANGES:

; Registers A, D, E, H, L



; RETURNS:

; |   H    |    L   |

; | Port 2 | Port 1 |

; |xR21RLDU|xx21RLDU|

;

; Joystick Port 1

;         Bit 0 \

;         Bit 1  | Pad

;         Bit 2  | Dirrection

;         Bit 3 /

;         Bit 4    Button 1

;         Bit 5    Button 2

;         Bit 6    Not used

;         Bit 7    Not used

;

; Joystick Port 2

;         Bit 0 \

;         Bit 1  | Pad

;         Bit 2  | Dirrection

;         Bit 3 /

;         Bit 4    Button 1

;         Bit 5    Button 2

;         Bit 6    Reset

;         Bit 7    Not used

;

; Pad Dirrections:

;  0 = Not Pressed

;  1 = Up

;  2 = Down

;  4 = Left

;  5 = Up/Left

;  6 = Down/Left

;  8 = Right

;  9 = Up/Right

;  A = Down/Right

;

; Buttons:

;  0 = Not Pressed

;  1 = Button 1

;  2 = Button 2

;  3 = Both Buttons



JOYSTK: IN A, (0DCh)    ; Read Port 1

        CPL             ; Get rid of that Active Low junk

        LD L, A 

        LD E, A

        IN A, (0DDh)    ; Read Port 2

        CPL             ; Again no more Active Low

        LD H, A

        LD D, A

        ADD HL, DE      ; First 16-bit Shift Left Logical

        LD D, H

        LD E, L

        ADD HL, DE      ; Second 16-bit Shift Left Logical

        SRL L           ; Put the Low Byte back

        SRL L           ; with out effecting the High Byte

        LD A, H

        AND 01000000b   ; Check for Reset

        JP NZ, 0000h    ; Start over if Reset was pressed

        RET

