; Character Data Loader (Revision 1)

; Revision history:

;  0: Can only load 32 characters at a time.

;     Loads 16 color characters.

;  1: Can load 256 characters at a time.

;     Added support for 2 color characters.



; Requirements:

; HL Register set to the start of the character data.



; Character Data Format:        

; DB 31d ; Number of Characters

; DB 00h ; Low byte of the Video Address

; DB 40h ; High byte of the Video Address

; DB 00h ; Color or Mono (00h = Color, 0FFh = mono)



; Color Characters (32 bytes/character)



; 00F00000 = 20,20,20,20 \

; 0F0F0000 = 50,50,50,50  \

; F000F000 = 88,88,88,88   | The 32 bytes

; FFFFF000 = F8,F8,F8,F8   | for a color

; F000F000 = 88,88,88,88   | version of

; F000F000 = 88,88,88,88   | the letter "A"

; F000F000 = 88,88,88,88  /

; 00000000 = 00,00,00,00 /



; DB 20h ; Byte 0 of the first character

; DB 20h

; DB 20h

; DB 20h

; DB 50h

; DB 50h

; DB 50h

; DB 50h

; DB 88h

; DB 88h

; DB 88h

; DB 88h

; DB 0F8h

; DB 0F8h

; DB 0F8h

; DB 0F8h

; DB 88h

; DB 88h

; DB 88h

; DB 88h

; DB 88h

; DB 88h

; DB 88h

; DB 88h

; DB 88h

; DB 88h

; DB 88h

; DB 88h

; DB 00h

; DB 00h

; DB 00h

; DB 00h ; Byte 31 of the first character



; Mono Characters (8 bytes/character)



; 00100000 = 20 \

; 01010000 = 50  \

; 10001000 = 88   | The 8 bytes

; 11111000 = F8   | for a mono

; 10001000 = 88   | version of the

; 10001000 = 88   | same letter "A"

; 10001000 = 88  /

; 00000000 = 00 /



; DB 20h ; Byte 0 of the first character

; DB 50h

; DB 88h

; DB 0F8h

; DB 88h

; DB 88h

; DB 88h

; DB 00h ; Byte 7 of the first character





CHARS1  EQU 4000h

CHARS2  EQU 4010h



	LD HL, CHARS1

	CALL LDCHR

	LD HL, CHARS2

	CALL LDCHR

	HALT



LDCHR:  LD B, (HL)      ; Number of Characters

        INC HL

        LD A, (HL)      ; Low byte of Video Address

        OUT (0BFh), A

        INC HL

        LD A, (HL)      ; High byte of Video Address

        OUT (0BFh), A

        INC HL

        LD A, (HL)      ; 00h = 16 color FFh = mono

        AND A

        INC HL          ; Get next byte for the character

        JP NZ, MONO

COLOR:  PUSH BC         ; Save number of characters

        LD BC, 20BEh    ; 32 Bytes and Port 0BEh

        OTIR            ; Copy the full character to VRAM

	POP BC

        DJNZ, COLOR     ; Next character if there is one

        RET

MONO:   PUSH BC         ; Almost the same as for 16 color

        LD BC, 08BEh    ; Only 8 bytes this time

        LD A, (HL)

NXMCHR: OUT (0BEh), A   ; An expanded loop to speed processing

        OUT (0BEh), A   ; Loads the same byte 4 times

        OUT (0BEh), A   ; Even though you can store 1/4 the data

        OUTI            ; the SMS still needs 32 bytes/char

        JP NZ, NXMCHR

	POP BC

        DJNZ, MONO

        RET





; SAMPLE DATA



	ORG CHARS1

        DB 01d          ; Number of Characters

        DB 00h          ; Low byte of the Video Address

        DB 40h          ; High byte of the Video Address

        DB 0FFh         ; Mono

        DB 20h          ; Byte 0 of the first character

	DB 50h

	DB 88h

	DB 0F8h

	DB 88h

	DB 88h

	DB 88h

        DB 00h          ; Byte 7 of the first character



	ORG CHARS2

        DB 01d          ; Number of Characters

        DB 00h          ; Low byte of the Video Address

        DB 40h          ; High byte of the Video Address

        DB 00h          ; Color

        DB 20h          ; Byte 0 of the first character

	DB 20h

	DB 20h

	DB 20h

	DB 50h

	DB 50h

	DB 50h

	DB 50h

	DB 88h

	DB 88h

	DB 88h

	DB 88h

	DB 0F8h

	DB 0F8h

	DB 0F8h

	DB 0F8h

	DB 88h

	DB 88h

	DB 88h

	DB 88h

	DB 88h

	DB 88h

	DB 88h

	DB 88h

	DB 88h

	DB 88h

	DB 88h

	DB 88h

	DB 00h

	DB 00h

	DB 00h

        DB 00h          ; Byte 31 of the first character



