        RESET	EQU 0000h

        VBLANK	EQU 0038h

        PAUSE	EQU 0066h

        START	EQU 0080h

        FOOTER	EQU 7FF0h

        PAL1	EQU 6000h

        CHARS1	EQU 8000h



RESET:	ORG RESET

        DI              ; Disable Interupts

	LD SP, 0C200h	; Set stack pointer

	IM 1		; Set Interupt Mode

	JP START



VBLANK:	ORG VBLANK

        RET



PAUSE:	ORG PAUSE

	RETN



START:	ORG START

        LD A, 81h

        OUT (0BFh), A

        LD A, 00000000b

        OUT (0BFh), A   ; Disable screen

        LD HL, 0C000h

	LD DE, 0C001h

	LD (HL), 0

	LD BC, 1FFEh

	LDIR		; Zero RAM contents

        LD HL, PAL1

        CALL LDPAL

        LD HL, CHARS1

        CALL LDCHR

        LD A, 00h

        OUT (0BFh), A

        LD A, 78h

        OUT (0BFh), A   ; Point to character screen

        LD A, 00h

        OUT (0BEh), A

        OUT (0BEh), A   ; Output char 0 with status byte 0

        LD A, 81h

        OUT (0BFh), A

        LD A, 01100000b

        OUT (0BFh), A   ; Enable screen

HOLD:   NOP

        JP HOLD



        

LDPAL:  LD A, (HL)      ; Point A to the first byte of the palette

        AND A           ; Check for 00h

        JP Z, CHPAL

        CPL             ; FFh will turn into 00h

        AND A           ; Check to see if it was FFh

        JP Z, SPPAL

        LD D, H

	LD E, L		; Save HL for CHPAL to use

        CALL SPPAL      ; Call SPPAL cause I can use CHPAL's RET

        LD H, D

	LD L, E		; Get HL back and run right into CHPAL

CHPAL:  LD A, 00h

	OUT (0BFh), A	; Output to the Character Palette

	LD A, 0C0h

        OUT (0BFh), A

        INC HL          ; Now points to Palette Entry 0

        LD BC, 0FBEh    ; 16 Entries/Port 0BEh

        OTIR            ; Dump all 16 to the VRAM

        RET

SPPAL:  OUT (0BFh), 10h ; Output to the Sprite Palette

        OUT (0BFh), 0C0h

        INC HL          ; Now points to Palette Entry 0

        LD BC, 0FBEh    ; 16 Entries/Port 0BEh

        OTIR            ; Dump all 16 to the VRAM

        RET



LDCHR:  LD B, (HL)      ; Number of Characters

        INC HL

        LD A, (HL)      ; Low byte of Video Address

        OUT (0BFh), A

        INC HL

        LD A, (HL)      ; High byte of Video Address

        OUT (0BFh), A

        INC HL

        LD A, (HL)      ; 00h = 16 color FFh = mono

        AND A

        INC HL          ; Get next byte for the character

        JP NZ, MONO

COLOR:  PUSH BC         ; Save number of characters

        LD BC, 20BEh    ; 32 Bytes and Port 0BEh

        OTIR            ; Copy the full character to VRAM

	POP BC

        DJNZ, COLOR     ; Next character if there is one

        RET

MONO:   PUSH BC         ; Almost the same as for 16 color

        LD BC, 08BEh    ; Only 8 bytes this time

        LD A, (HL)

NXMCHR: OUT (0BEh), A   ; An expanded loop to speed processing

        OUT (0BEh), A   ; Loads the same byte 4 times

        OUT (0BEh), A   ; Even though you can store 1/4 the data

        OUTI            ; the SMS still needs 32 bytes/char

        JP NZ, NXMCHR

	POP BC

        DJNZ, MONO

        RET



	ORG PAL1

        DB 00h         ; Palette Type (Both)

        DB 00000000b   ; Entry 00h (Black)

        DB 00000011b   ; Entry 01h (Red)

        DB 00001100b   ; Entry 02h (Green)

        DB 00110000b   ; Entry 03h (Blue)

        DB 00000010b   ; Entry 04h (Medium Red)

        DB 00001000b   ; Entry 05h (Medium Green)

        DB 00100000b   ; Entry 06h (Medium Blue)

        DB 00000001b   ; Entry 07h (Dark Red)

        DB 00000100b   ; Entry 08h (Dark Green)

        DB 00010000b   ; Entry 09h (Dark Blue)

        DB 00101010b   ; Entry 0Ah (Light Gray)

        DB 00010101b   ; Entry 0Bh (Dark Gray)

        DB 00111100b   ; Entry 0Ch (Cyan)

        DB 00110011b   ; Entry 0Dh (Magenta)

        DB 00001111b   ; Entry 0Eh (Yellow)

        DB 00111111b   ; Entry 0Fh (White)



	ORG CHARS1

        DB 01d          ; Number of Characters

        DB 00h          ; Low byte of the Video Address

        DB 40h          ; High byte of the Video Address

        DB 0FFh         ; Mono

        DB 20h          ; Byte 0 of the first character

        DB 50h

        DB 88h

        DB 0F8h

        DB 88h

        DB 88h

        DB 88h

        DB 00h          ; Byte 7 of the first character



	ORG FOOTER

        DB "TMR SMSC"  ; Super Majik Spiral Crew Tradmark

        DW 9719h       ; Year

        DW 0FFFFh      ; Checksum

        DW 0040h       ; Part Num (Card)

        DW 0FFFFh      ; ? (Filler)

