; The first working program released by the Super Majik Spiral Crew!

; Coded by Neon Spiral Injector

; Based on a Game Gear sample program by Charles Doty

; This program just loads my monster 256 character, character set

; and sets the first 2 colors of the character palette and then displays

; some text on the screen.

; I'm hoping to add a lot more to this demo in the future.

; Well anyway check it out!



#DEFINE RESET   $0000

#DEFINE VBLANK  $0038

#DEFINE PAUSE   $0066

#DEFINE START   $0080

#DEFINE FOOTER  $7FF0



.ORG  RESET

        DI              ; Disable Interrupts

        LD SP,$C200     ; Set stack pointer

        IM   1          ; Interrupt Mode 1

        JR START



.ORG START              ; Main Program

        LD   HL,VDPRegsTable ;Sets hl to point to VDP register values.

        LD   B,$0B      ;Sets first 11 VDP registers. Starting at 0.

        LD   C,$80      ;High byte is register number + $80

VDPRegsLoop:

        LD   A,(HL)

        INC  HL

        OUT  ($BF),A    ;$BF selects VDP($80) register,CRAM(c0h) or VRAM(40h). 

        LD   A,C        ;Transfer data first then register number

        INC  C

        OUT  ($BF),A    ;Selects VDP register.

        DEC  B

        JR   NZ,VDPRegsLoop



        LD   BC,$4000   ;Transfer 4000h 0's to VRAM.

ClearVRAMLoop:

        SUB  A

        OUT  ($BE),A

        DEC  BC

        LD   A,B

        OR   C

        JR   NZ,ClearVRAMLoop



        LD HL,PalData

        CALL LDPAL



        LD HL,CharData

        CALL LDCHR



    ld   a,$00      ;Setup VRAM for write to $3800

    out  ($bf),a

    ld   a,$78

    out  ($bf),a



    ld   hl,ScreenData  ;Transfer Tile numbers to VRAM.

    ld   bc,ScreenCount

ScreenLoop:

    ld   a,(hl)

    inc  hl

    out  ($be),a

    sub  a

    out  ($be),a

    dec  bc

    ld   a,b

    or   c

    jr   nz,ScreenLoop



    ld   a,$c0  ;Turn on the Screen

    out  ($bf),a

    ld   a,$81

    out  ($bf),a



WaitforEndofScreen:

    in   a,($7e)

    cp   $c0

    jr   nz,WaitforEndofScreen



EndlessLoop:

    jr   EndlessLoop



; ==========================================================================

; Sub-Routines

; ==========================================================================



LDPAL:  LD   A,(HL)       ; Point A to the first byte of the palette

        AND  A           ; Check for $00

        JR   Z,CHPAL

        CPL             ; $FF will turn into $00

        AND  A           ; Check to see if it was $FF

        JR   Z,SPPAL

        LD   D,H          ; Save HL for CHPAL to use

        LD   E,L

        CALL SPPAL      ; Call SPPAL cause I can use CHPAL's RET

        LD   H,D          ; Get HL back and run right into CHPAL

        LD   L,E

CHPAL:  LD   A,$00

        OUT  ($BF),A     ; Output to the Character Palette

        LD   A,$C0

        OUT  ($BF),A

        INC  HL          ; Now points to Palette Entry 0

        LD   BC,$0FBE     ; 16 Entries/Port $BE

        OTIR            ; Dump all 16 to the VRAM

        RET

SPPAL:  LD   A,$10

        OUT  ($BF),A     ; Output to the Sprite Palette

        LD   A,$C0

        OUT  ($BF),A

        INC  HL          ; Now points to Palette Entry 0

        LD   BC,$0FBE     ; 16 Entries/Port 0BEh

        OTIR            ; Dump all 16 to the VRAM

        RET



LDCHR:  LD   B,(HL)       ; Number of Characters

        INC  HL

        LD   A,(HL)       ; Low byte of Video Address

        OUT  ($BF),A

        INC  HL

        LD   A,(HL)       ; High byte of Video Address

        OUT  ($BF),A

        INC  HL

        LD   A,(HL)       ; 00 = 16 color FF = mono

        INC  HL          ; Get next byte for the character

        AND  A           ; Check color mode

        JR   NZ,MONO

COLOR:  PUSH BC         ; Save number of characters

        LD   BC,$20BE     ; 32 Bytes and Port $BE

        OTIR            ; Copy the full character to VRAM

        POP  BC

        DJNZ COLOR      ; Next character if there is one

        RET

MONO:   PUSH BC         ; Almost the same as for 16 color

        LD   B,$08        ; Only 8 bytes this time

NXMCHR: LD   A,(HL)

        OUT  ($BE),A

        SUB  A

        OUT  ($BE),A     ; Load 00s into the other 3 bit planes

        OUT  ($BE),A

        OUT  ($BE),A

        INC  HL

        DEC  B

        JR   NZ, NXMCHR

        POP  BC

        DJNZ MONO

        RET



; ==========================================================================

; Data

; ==========================================================================



VDPRegsTable:

    .byte    $06,$80,$ff,$ff,$ff,$ff,$ff,$00,$00,$00,$00



ScreenData:

 .byte  " "

 .byte  "This is the second attempt by  "

 .byte  " "

 .byte  "the Super Majik Spiral Crew to "

 .byte  " "

 .byte  "display text to the SMS screen "

 .byte  " "

 .byte  "the first was a failure but we "

 .byte  " "

 .byte  "got it this time!              "

 .byte  " "

 .byte  "                               "

 .byte  " "

 .byte  "Code and Font by:              "

 .byte  " "

 .byte  "  Neon Spiral Injector         "

 .byte  " "

 .byte  "Very special thanks to:        "

 .byte  " "

 .byte  "  Apollo                       "

 .byte  " "

 .byte  "Mental Support from:           "

 .byte  " "

 .byte  "  Perry S. Nightmare           "

 .byte  " "

 .byte  "  and Razor Grrrl              "

 .byte  " "

 .byte  "Couldn't of done it without:   "

 .byte  " "

 .byte  "  James McKay - Massage Rules! "

 .byte  " "

 .byte  "  Charles Doty - Basic GG Code "

 .byte  " "

 .byte  "                               "

 .byte  " "

 .byte  "Be sure to check out the SMSC  "

 .byte  " "

 .byte  "web site!  Coming soon to:     "

 .byte  "                               "

 .byte  " "

 .byte  "h"

 .byte  "ttp://stu.ac.cc.md.us/~cmeadors"

 .byte  " "

 .byte  "                               "

 .byte  " "

 .byte  "Placed in the public domain by "

 .byte  " "

 .byte  "The SMSC on 06/06/1997         "

ScreenEnd:

ScreenCount = ScreenEnd-ScreenData



PalData:

.include pals.asm

PalEnd:

PalCount = PalEnd-PalData



CharData:

.include chars.asm

CharEnd:

CharCount = CharEnd-CharData



.ORG FOOTER

        .DB "TMR SMSC"  ; Super Majik Spiral Crew Tradmark

        .DW 9719h       ; Year

        .DW 0D23Fh      ; Checksum

        .DW 4002h       ; Part Num (Card) SMSC's second release

        .DW 0606h       ; Day / Month



.END

